import arcpy
from arcpy import env
from numpy import*


class ConfMatrix(object):
    """Accuracy assessment."""
    def __init__(self):
        self.label = "Accuracy assessment"
        self.description =  ("Calculates matrix of confusion ")
        self.canRunInBackground = False
    def getParameterInfo(self):
        # Input parameter 1
        in_dirr = arcpy.Parameter(
            displayName="Workspace",
            name="in_dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter 2
        in_shp = arcpy.Parameter(
            displayName="Segments test shapefile  (.shp)",
            name="in_shp",
            datatype="GPTableView",
            parameterType="Required",
            direction="Input")
        # Input parameter 3
        s_pole = arcpy.Parameter(
            displayName="Field of results class",
            name="s_pole",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        # Input parameter 4       
        s_pole.filter.list = ['Short', 'Long']
        s_pole.parameterDependencies = [in_shp.name]
        r_pole = arcpy.Parameter(
            displayName="Field of reference class",
            name="r_pole",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        r_pole.filter.list = ['Short', 'Long']
        r_pole.parameterDependencies = [in_shp.name]

        parameters=[in_dirr,in_shp,s_pole,r_pole]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):
       
        wdir=parameters[0].valueAsText   # data
        testlayer=parameters[1].valueAsText   # data
        field_class=parameters[2].valueAsText   # data
        field_reff=parameters[3].valueAsText   # data

        
        cc=zeros((1000,2),int)
        confmatrix=zeros((50,50),int)
        classy1=zeros((1000),int)
        for i in range(0,1000):
            classy1[i]=10000

        classy2=zeros((100),int)
        liczba_rek=0

        env.workspace=wdir

        with arcpy.da.SearchCursor(testlayer,[field_class,field_reff]) as cur:
            for row in cur:
                k1=row[0];k2=row[1]
                cc[liczba_rek,0]=k1
                cc[liczba_rek,1]=k2
                #print k1,k2
                liczba_rek+=1
                classy1[k1]=1;classy1[k2]=1
               
        del cur

        # przypisanie klas


        llclas=0
        for i in range(0,1000):
            if classy1[i]==1:
                classy2[llclas]=i
                llclas+=1
                
        for i in range(0,llclas):
            k=classy2[i]
            classy1[k]=i
          

        # wypelnianie confusion matrix
        for i in range(0,liczba_rek):
            iref=cc[i,1]
            icla=cc[i,0]
            confmatrix[classy1[iref],classy1[icla]]+=1

        # Obliczenie sumy po przekatnej sumCjj
        sumCjj=0.0
        for i in range(0,llclas):
            sumCjj=sumCjj+confmatrix[i,i]


        cmcal=zeros((llclas,4),float)  #0 suma po kol Ncj, 1 -suma po wier Nri, 2 - Cjj/Ncj, 3-Cjj/Nr

        # zliczanie po kolumnach  indeks 0
        for i in range(0,llclas):
            sum=0
            for j in range(0,llclas):
                sum=sum+confmatrix[j,i]
            cmcal[i,0]=sum
            cmcal[i,2]=float(confmatrix[i,i])/sum*100
        # zliczanie po wierszach  indeks 1
        for i in range(0,llclas):
            sum=0
            for j in range(0,llclas):
                sum=sum+confmatrix[i,j]
            cmcal[i,1]=sum
            cmcal[i,3]=float(confmatrix[i,i])/sum*100
            

        # obliczenie kappa: sumCjj
        sumNrNk=0
        for i in range(0,llclas):
            sumNrNk=cmcal[i,0]*cmcal[i,1]
            
        N=liczba_rek

        kappa=(N*sumCjj-sumNrNk)/(N*N-sumNrNk)*100

        ss=""
        for i in range (0,llclas):
            if i==0:
                ss=ss+"           Class "+str(classy2[i])
            else:
                ss=ss+" Class "+str(classy2[i])
        ss=ss+"  Row sum Nri  Cii/Nri(%)"  
        arcpy.AddMessage(ss)
        for i in range (0,llclas):
            ss="Reference "+str(classy2[i])+"    "
            for j in range (0,llclas):
                if j==0:
                    ss=ss+str(confmatrix[i,j])
                else:
                    ss=ss+"       "+str(confmatrix[i,j])
            ss=ss+"     "+str(int(cmcal[i,1]))+"        "+str(round(cmcal[i,3],2))    
            arcpy.AddMessage(ss)
        ss="Column sum Ncj "
        for i in range (0,llclas):
            ss=ss+str(int(cmcal[i,0]))+"      "
        arcpy.AddMessage(ss)   
        ss="   Cjj/Ncj (%) "
        for i in range (0,llclas):
            ss=ss+str(round(cmcal[i,2],2))+"   "
        ss=ss+"    Kappa:  "+str(round(kappa,2))+"  %"
        arcpy.AddMessage(ss)     