import arcpy
from arcpy import env

wspace=""    # katalog roboczy
in_poly=""
pole=""   # mapa klas
maval=0
out_poly=""

class MergeObjects(object):
    """Zonal Statistics ."""
    def __init__(self):
        self.label = "Merge objects"
        self.description =  ("Merge objects with the same class")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
##        # Input parameter
##        in_dirr = arcpy.Parameter(
##            displayName="Workspace",
##            name="in_dirr",
##            datatype="DEWorkspace",
##            parameterType="Required",
##            direction="Input")
        # Input parameter
        in_shp = arcpy.Parameter(
            displayName="Segments shapefile  (.shp)",
            name="in_shp",
            datatype="GPTableView",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_pole = arcpy.Parameter(
            displayName="Field of classes to merge",
            name="in_pole",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        in_pole.filter.list = ['Short', 'Long']
        in_pole.parameterDependencies = [in_shp.name]
        # Input parameter
        in_mm = arcpy.Parameter(
            displayName="Class value to merge",
            name="in_mm",
            datatype="GPLong",
            parameterType="Required",
            direction="Input")
        # Output parameter
        out_shp = arcpy.Parameter(
            displayName="Segments shapefile  (.shp)",
            name="out_shp",
            datatype="DEShapefile",
            parameterType="Required",
            direction="Output")

        parameters=[in_shp,in_pole,in_mm,out_shp]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):





        #wspace=parameters[0].valueAsText    # katalog roboczy      
        in_poly=parameters[0].valueAsText   # data
        pole=parameters[1].valueAsText   # data
        mval=parameters[2].value
        out_poly=parameters[3].valueAsText


        #env.workspace=wspace

        where_clause='"'+pole+'"='+str(mval)
        arcpy.Select_analysis(in_poly, "t1.shp", where_clause)
        where_clause='"'+pole+'"<>'+str(mval)
        arcpy.Select_analysis(in_poly, "t2.shp", where_clause)

        arcpy.Dissolve_management("t1.shp","t1d.shp",pole)
        arcpy.MultipartToSinglepart_management("t1d.shp","t1ds.shp")
        arcpy.CalculateField_management("t1ds.shp", "ORIG_FID", '!FID!', "PYTHON_9.3")
        arcpy.AddField_management("t1ds.shp", "PPP", "DOUBLE")
        arcpy.CalculateField_management("t1ds.shp", "PPP", 'float(!SHAPE.area!)', "PYTHON_9.3")

        arcpy.Intersect_analysis (["t1.shp","t1ds.shp"],"t12.shp")
        arcpy.AddField_management("t12.shp", "PPP2", "DOUBLE")
        arcpy.CalculateField_management("t12.shp", "PPP2", 'float(!SHAPE.area!)', "PYTHON_9.3")

        fieldNameList = [field.name for field in arcpy.ListFields("t1.shp")]


        fieldNameList.remove(pole)
        fieldNameList.remove('FID')
        fieldNameList.remove('Shape')
        #fieldNameList.remove('ID')
        fieldNameList.remove('OBID')


        for field in fieldNameList:
            sss=str(field)
            #print sss
            expr="float(!"+sss+"!*(!PPP2!/!PPP!))"
            #print expr
            arcpy.CalculateField_management("t12.shp", sss, expr, "PYTHON_9.3")    

        statf=[]
        n=len(fieldNameList)
        
        for i in range(0,n):
            tt=[]
            tt.append(fieldNameList[i])
            tt.append("SUM")
            statf.append(tt)

        

        arcpy.Statistics_analysis("t12.shp", "stat.dbf", statf, "ORIG_FID")

        llstat=[]
        for i in range(0,n):
            llstat.append("SUM_"+fieldNameList[i])


        arcpy.JoinField_management("t1ds.shp","ORIG_FID","stat.dbf","ORIG_FID",llstat)

        i=0
        for field in fieldNameList:
           arcpy.AddField_management("t1ds.shp", field, "DOUBLE")
           p1=llstat[i]
           i+=1
           expr="!"+p1+"!"
           arcpy.CalculateField_management("t1ds.shp", field, expr, "PYTHON_9.3")    

        for field in llstat:
           arcpy.DeleteField_management("t1ds.shp",field)
        arcpy.DeleteField_management("t1ds.shp","PPP")
        arcpy.DeleteField_management("t1ds.shp","ORIG_FID")

        arcpy.Merge_management(["t2.shp","t1ds.shp"],out_poly)
        expr="!FID!"
        arcpy.CalculateField_management(out_poly,"OBID", expr, "PYTHON_9.3") 
        arcpy.Delete_management("t1.shp")
        arcpy.Delete_management("t2.shp")
        arcpy.Delete_management("t1d.shp")
        arcpy.Delete_management("t1ds.shp")
        arcpy.Delete_management("t12.shp")
        arcpy.Delete_management("stat.dbf")

          