#Narzedzie do obliczania parametrow tekstury 
import arcpy
import math
from arcpy import env
from arcpy.sa import*
from numpy import*

class Texture(object):
    """Zonal Statistics ."""
    def __init__(self):
        self.label = "Texture"
        self.description =  ("Calculate Texture")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
        # Input parameter
        in_dirr = arcpy.Parameter(
            displayName="Workspace",
            name="in_dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_shp = arcpy.Parameter(
            displayName="Segments shapefile name without path (.shp)",
            name="in_shp",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_val = arcpy.Parameter(
            displayName="Raster image name without path (.tif)",
            name="in_val",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_ch = arcpy.Parameter(
            displayName="Field name ending",
            name="in_ch",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        in_ch.value=1

        parameters=[in_dirr,in_shp,in_val,in_ch]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):





        wspace=parameters[0].valueAsText    # katalog roboczy      
        ftab=parameters[1].valueAsText
        vch=parameters[2].valueAsText   # data
        sch=parameters[3].valueAsText


        env.workspace=wspace


        env.overwriteOutput=True


        ### wczytanie zdjecia i utworzenie rastra


        chv=Raster(vch)
        outRaster = "xxx.tif"
        cellSize = chv.meanCellWidth

        # Execute PolygonToRaster
        arcpy.PolygonToRaster_conversion(ftab, "OBID", outRaster, "MAXIMUM_AREA","", cellSize)
        rseed=Raster(outRaster)


        arcpy.AddMessage("Wczytanie i konwersja OK")

        # zapisanie info o georeferencji
        sssref=rseed.spatialReference.exporttostring()
        cx=rseed.meanCellWidth
        cy=rseed.meanCellHeight
        licx=rseed.extent.XMin
        licy=rseed.extent.YMin
        point=arcpy.CreateObject("point")
        point.X=licx
        point.Y=licy
        ##
        ##
        ### zamiana na numpy

        seed=arcpy.RasterToNumPyArray(rseed)
        ch=arcpy.RasterToNumPyArray(chv)
        ### liczba wierszy i kolumn rastrow
        cc=seed.shape[1]
        rr=seed.shape[0]
        klasy=amax(seed)


        # wypelnianie tablic zakrsow i wart. min i max

        tabex=zeros((klasy+1,4),int)     #xmin,xmax,ymin,ymax
        tabmm=zeros((klasy+1,2),float)   #vmin,vmax

        for i in range (0,klasy+1):
            for j in range (0,4):
                tabex[i,j]=-1
                

        for i in range(0,rr):
            for j in range(0,cc):
                klas=seed[i,j]
                if tabex[klas,0]==-1:
                    tabex[klas,0]=j
                else:
                    if tabex[klas,0]>j:
                        tabex[klas,0]=j
                if tabex[klas,1]==-1:
                    tabex[klas,1]=j
                else:
                    if tabex[klas,1]<j:
                        tabex[klas,1]=j
                if tabex[klas,2]==-1:
                    tabex[klas,2]=i
                else:
                    if tabex[klas,2]>i:
                        tabex[klas,2]=i
                if tabex[klas,3]==-1:
                    tabex[klas,3]=i
                else:
                    if tabex[klas,3]<i:
                        tabex[klas,3]=i
                if tabmm[klas,0]<0.00001:
                    tabmm[klas,0]=ch[i,j]
                else:
                    if tabmm[klas,0]>ch[i,j]:
                        tabmm[klas,0]=ch[i,j]
                if tabmm[klas,1]<0.00001:
                    tabmm[klas,1]=ch[i,j]
                else:
                    if tabmm[klas,1]<ch[i,j]:
                        tabmm[klas,1]=ch[i,j]






        def reskalowanie(vv,kk,vmin,vmax):
            ''' Zaklada ze vv (float) >0, kk (int) >3, vmin, vmax >0 i vmax>vmin
                zwraca reskalowana vv do (int) w przedziale 1 - kk'''
            if abs(vmax-vmin)<0.00001:
                dd=0.0
            else:
                dd=(vv-vmin)/(vmax-vmin)
            t1=dd*kk-0.5
            if t1>10000.0:
                t1=0.0
            if t1<0.00001:
                t1=0.0
            t2=int(round(t1))
            if t2<0:
                t2=0
            if t2==kk:
                t2=kk-1
            t2=t2+1
            return t2



        gsize=8
        texture=zeros((klasy+1,8),float)   #oobliczone tekstury dla obiektu kk
         
        for kk in range (1,klasy+1):
            icc=tabex[kk,1]-tabex[kk,0]
            irr=tabex[kk,3]-tabex[kk,2]


            texim=zeros((irr+1,icc+1),int)

            glcmH=zeros((gsize,gsize),int)

            glcm=zeros((gsize,gsize),float)

            for j in range(0,irr+1):
                for i in range(0,icc+1):
                    ixx=tabex[kk,0]+i
                    jyy=tabex[kk,2]+j
                    if seed[jyy,ixx]==kk:
                        texim[j,i]=reskalowanie(ch[jyy,ixx],gsize,tabmm[kk,0],tabmm[kk,1])
                    
                

            # H E    
            for j in range(0,irr+1):
                for i in range(0,icc):
                    if texim[j,i]>0:
                        ref=texim[j,i]-1
                        ngb=texim[j,i+1]-1
                        glcmH[ref,ngb]+=1
            # H W
            for j in range(0,irr+1):
                for i in range(1,icc+1):
                    if texim[j,i]>0:
                        ref=texim[j,i]-1
                        ngb=texim[j,i-1]-1
                        glcmH[ref,ngb]+=1
            # V S
            for j in range(0,irr):
                for i in range(0,icc+1):
                    if texim[j,i]>0:
                        ref=texim[j,i]-1
                        ngb=texim[j+1,i]-1
                        glcmH[ref,ngb]+=1

            # V N
            for j in range(1,irr+1):
                for i in range(0,icc+1):
                    if texim[j,i]>0:
                        ref=texim[j,i]-1
                        ngb=texim[j-1,i]-1
                        glcmH[ref,ngb]+=1

            # GLCM normalization

            suma=0.0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    suma=suma+glcmH[i,j]

            sum=0 
            for j in range(0,gsize):
                for i in range(0,gsize):
                    glcm[i,j]=glcmH[i,j]/float(suma)
                    sum=glcm[i,j]+sum


           
            # Contrast  - 0
            pp=0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=(glcm[j,i]*(j-i)*(j-i))+pp
            texture[kk,0]=pp

            # Disimilarity  - 1
            pp=0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=(glcm[j,i]*math.fabs(j-i))+pp
            texture[kk,1]=pp

            # Homogenity  - 2
            pp=0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=(glcm[j,i]/(1+(i-j)**2))+pp
            texture[kk,2]=pp

          # Energy  - 3
            pp=0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=((glcm[j,i])**2)+pp
            texture[kk,3]=math.sqrt(pp)

          # Entropy  - 4
            pp=0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    if glcm[j,i]==0:
                        ll=0.0
                    else:
                        ll=-1.0*math.log(glcm[j,i])
                    pp=(glcm[j,i]*ll)+pp
            texture[kk,4]=pp
            
            # GLCM mean - 5
            
            pp=0
            ppn=0.0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=((glcm[j,i])*j)+pp
            ppn=pp
            
            pp=0
            ppn=0.0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=((glcm[j,i])*i)+pp
            ppr=pp  

            texture[kk,5]=0.5*(ppn+ppr)
             # GLCM std - 6
            
            pp=0
            psn=0.0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=((glcm[j,i])*(j-ppn)**2)+pp
            psn=pp
            
            pp=0
            psr=0.0
            for j in range(0,gsize):
                for i in range(0,gsize):
                    pp=((glcm[j,i])*(i-ppr)**2)+pp
            psr=pp

            texture[kk,6]=math.sqrt(0.5*(psn+psr))
            # GLCM correlation - 7
            if psn==0 or psr==0:
                texture[kk,7]=0
            else:
                pp=0
                for j in range(0,gsize):
                    for i in range(0,gsize):
                        pp=((glcm[j,i])*(((j-ppn)*(i-ppr))/math.sqrt(psn*psr)))+pp
                
                texture[kk,7]=pp
            #print kk, texture[kk,0],texture[kk,1],texture[kk,2],texture[kk,3],texture[kk,4],texture[kk,5], texture[kk,6],texture[kk,7]

        # dodajemy pola

        f1="CON"+sch
        f2="DIS"+sch
        f3="HOM"+sch
        f4="ENER"+sch
        f5="ENT"+sch
        f6="GMEAN"+sch
        f7="GSTD"+sch
        f8="GCORR"+sch

        arcpy.AddField_management(ftab,f1,"DOUBLE")   # Contrast - 0
        arcpy.AddField_management(ftab,f2,"DOUBLE")   # Disimilarity - 1
        arcpy.AddField_management(ftab,f3,"DOUBLE")   # Homogenity - 2
        arcpy.AddField_management(ftab,f4,"DOUBLE")   # Energy - 3
        arcpy.AddField_management(ftab,f5,"DOUBLE")   # Entropy - 4
        arcpy.AddField_management(ftab,f6,"DOUBLE")   # GLCM Mean - 5
        arcpy.AddField_management(ftab,f7,"DOUBLE")     # GLCM std - 6
        arcpy.AddField_management(ftab,f8,"DOUBLE")    # GLCM Correlation - 7

        with arcpy.da.UpdateCursor(ftab,["OBID",f1,f2,f3,f4,f5,f6,f7,f8]) as cur:
            for row in cur:
                kk=row[0]
                row[1]=texture[kk,0]
                row[2]=texture[kk,1]
                row[3]=texture[kk,2]
                row[4]=texture[kk,3]
                row[5]=texture[kk,4]
                row[6]=texture[kk,5]
                row[7]=texture[kk,6]
                row[8]=texture[kk,7]
                cur.updateRow(row)
        del cur       

        arcpy.Delete_management(outRaster)    