# Segmentation Landsat8 wersja 1.2  12.2014
# Using hybrid linkage region growing algorithms
# Modified algorithm by:  Devereux, Amable, Costa Posada An efficient segmentation algorithm for
# lanscape analysis. Int. Journal of Applied Earth Obs. and Geoinfo. 6 (2004) 47-61


import arcpy
import math
from arcpy import env
from arcpy.sa import*
from numpy import*
import os

class Segmentation(object):
    """Przeprowadza segmentacje obrazu."""
    def __init__(self):
        self.label = "Segmentation"
        self.description =  ("Perform segmentation of landsat project images.")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
        # Input parameter
        in_dirr = arcpy.Parameter(
            displayName="Workspace",
            name="in_dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_date = arcpy.Parameter(
            displayName="Date (middle part of channals name format: yyyy-mm-dd)",
            name="in_date",
            datatype="GPstring",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_ppp = arcpy.Parameter(
            displayName="Project name (end part of channals name)",
            name="in_ppp",
            datatype="GPstring",
            parameterType="Required",
            direction="Input")
        # Output parameter
        out_edge = arcpy.Parameter(
            displayName="Edge raster name without path (.tif)",
            name="out_edge",
            datatype="GPString",
            parameterType="Required",
            direction="Output")
        # Output parameter
        out_seg = arcpy.Parameter(
            displayName="Segments polygons name without path [.shp] ",
            name="out_seg",
            datatype="GPString",
            parameterType="Required",
            direction="Output")
        # Input parameter
        in_bedge = arcpy.Parameter(
            displayName="Edge test only",
            name="in_bedge",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        in_bedge.value=True
        # Input parameter
        in_th = arcpy.Parameter(
            displayName="Edhe treshold",
            name="in_th",
            datatype="GPLong",
            parameterType="Required",
            direction="Input")
        in_th.value=30
        # Input parameter
        in_ch1 = arcpy.Parameter(
            displayName="Ch1 weight (any positiv value) ",
            name="in_ch1",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch1.value=1.0
        # Input parameter
        in_ch2 = arcpy.Parameter(
            displayName="Ch2 weight (any positiv value) ",
            name="in_ch2",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch2.value=1.0
        # Input parameter
        in_ch3 = arcpy.Parameter(
            displayName="Ch3 weight (any positiv value) ",
            name="in_ch3",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch3.value=1.0
        # Input parameter
        in_ch4 = arcpy.Parameter(
            displayName="Ch4 weight (any positiv value) ",
            name="in_ch4",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch4.value=1.0
        # Input parameter
        in_ch5 = arcpy.Parameter(
            displayName="Ch5 weight (any positiv value) ",
            name="in_ch5",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch5.value=1.0
        # Input parameter
        in_ch6 = arcpy.Parameter(
            displayName="Ch6 weight (any positiv value) ",
            name="in_ch6",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch6.value=1.0
        # Input parameter
        in_ch7 = arcpy.Parameter(
            displayName="Ch7 weight (any positiv value) ",
            name="in_ch7",
            datatype="GPDouble",
            parameterType="Required",
            direction="Input")
        in_ch7.value=1.0
        
        parameters=[in_dirr,in_date,in_ppp,out_edge,out_seg,in_bedge,in_th,in_ch1,in_ch2,in_ch3,in_ch4,in_ch5,in_ch6,in_ch7]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):


        # parametry definiowane

        wspace=parameters[0].valueAsText    # katalog roboczy      
        data=parameters[1].valueAsText     # data
        projekt=parameters[2].valueAsText  # projekt
        mapedge=parameters[3].valueAsText   # mapa krawedzi
        mapklas=parameters[4].valueAsText   # mapa klas
        edgestop2=parameters[5].value   # stop program po krawedziach
        th=parameters[6].value       # treshold wykorzystywamy do tworzenia mapy krawedzi
        ww1=parameters[7].value     # wagi kanalow
        ww2=parameters[8].value 
        ww3=parameters[9].value 
        ww4=parameters[10].value 
        ww5=parameters[11].value 
        ww6=parameters[12].value 
        ww7=parameters[13].value 

        if edgestop2==0:
            edgestop=0
        else:
            edgestop=1
        
                
        tau=1.0     # wielokrotn std przy seeds (parametr staly)
##
##        # temp - okreslenie parametrow
##        wspace="D:\\LANDSAT8\\klastest2\\"
##        data="2013-09-06"
##        projekt="obia1"
##        mapedge="gran3"
##        mapklas="klas2"
##        edgestop=0
##        th=55

        # start programu
        # 1. obliczanie wag kanalow
        sww=float(ww1)+float(ww2)+float(ww3)+float(ww4)+float(ww5)+float(ww6)+float(ww7)

        ww1=ww1*7.0/sww
        ww2=ww2*7.0/sww
        ww3=ww3*7.0/sww
        ww4=ww4*7.0/sww
        ww5=ww5*7.0/sww
        ww6=ww6*7.0/sww
        ww7=ww7*7.0/sww
##        print "Wagi kanalow"
##        print "ch1", ww1
##        print "ch2", ww2
##        print "ch3", ww3
##        print "ch4", ww4
##        print "ch5", ww5
##        print "ch6", ww6
##        print "ch7", ww7
##        sys.stdout.flush()
##        time.sleep(0.5)

        # 2. wczytywanie nazw kanalow
        env.workspace=wspace

        env.overwriteOutput=True

        L8name=["L8"]*7   # 7 kanalow - 1,2,3,4,5,6,7

        csss="L8ch1rad_"+data+"_"+projekt+".tif"
        L8name[0]=csss
        csss="L8ch2rad_"+data+"_"+projekt+".tif"
        L8name[1]=csss
        csss="L8ch3rad_"+data+"_"+projekt+".tif"
        L8name[2]=csss
        csss="L8ch4rad_"+data+"_"+projekt+".tif"
        L8name[3]=csss
        csss="L8ch5rad_"+data+"_"+projekt+".tif"
        L8name[4]=csss
        csss="L8ch6rad_"+data+"_"+projekt+".tif"
        L8name[5]=csss
        csss="L8ch7rad_"+data+"_"+projekt+".tif"
        L8name[6]=csss


        # 3. wczytanie zdjec
        l8ch1=Raster(L8name[0])
        l8ch2=Raster(L8name[1])
        l8ch3=Raster(L8name[2])
        l8ch4=Raster(L8name[3])
        l8ch5=Raster(L8name[4])
        l8ch6=Raster(L8name[5])
        l8ch7=Raster(L8name[6])

        
        arcpy.AddMessage("Images input OK")



        # zapisanie info o georeferencji
        sssref=l8ch1.spatialReference.exporttostring()
        cx=l8ch1.meanCellWidth
        cy=l8ch1.meanCellHeight
        licx=l8ch1.extent.XMin
        licy=l8ch1.extent.YMin
        point=arcpy.CreateObject("point")
        point.X=licx
        point.Y=licy

        # zamiana rastrow na tablice numpy
        ch1=arcpy.RasterToNumPyArray(l8ch1)
        ch2=arcpy.RasterToNumPyArray(l8ch2)
        ch3=arcpy.RasterToNumPyArray(l8ch3)
        ch4=arcpy.RasterToNumPyArray(l8ch4)
        ch5=arcpy.RasterToNumPyArray(l8ch5)
        ch6=arcpy.RasterToNumPyArray(l8ch6)
        ch7=arcpy.RasterToNumPyArray(l8ch7)
        ### liczba wierszy i kolumn rastrow
        cc=ch1.shape[1]
        rr=ch1.shape[0]

        # 4. Filtracja Mean 5x5 dla pix>200000 i Gaussian smoothing dla mniejszych
        ccrr=cc*rr
        if ccrr>1:
            # Mean smoothing za pomoca okna 5x5
            neighborhood = NbrRectangle(5, 5, "CELL")
            ch1f=FocalStatistics(l8ch1, neighborhood, "MEAN")
            ch2f=FocalStatistics(l8ch2, neighborhood, "MEAN")
            ch3f=FocalStatistics(l8ch3, neighborhood, "MEAN")
            ch4f=FocalStatistics(l8ch4, neighborhood, "MEAN")
            ch5f=FocalStatistics(l8ch5, neighborhood, "MEAN")
            ch6f=FocalStatistics(l8ch6, neighborhood, "MEAN")
            ch7f=FocalStatistics(l8ch7, neighborhood, "MEAN")
        else:
            # Gaussian smoothing
            # kernel
            sigma=1.0
            suma=0
            ww=zeros((9,9),float)
            for i in range(0,9):
                for j in range(0,9):
                    xx=-4+i;yy=-4+j
                    ww[i,j]=math.exp(-(xx*xx+yy*yy)/(2*sigma*sigma))
                    suma=suma+ww[i,j]
            for i in range(0,9):
                for j in range(0,9):
                    ww[i,j]=ww[i,j]/suma
                    
            ##for i in range(0,9):     # to jest wydruk kernela
            ##    print ww[i,0],ww[i,1],ww[i,2],ww[i,3],ww[i,4],ww[i,5],ww[i,6],ww[i,7],ww[i,8]
            ##
            ##suma=0
            ##for i in range(0,9):
            ##    for j in range(0,9):
            ##        suma=ww[i,j]+suma
            ##print suma

            # filtracja
            ch1f=ch1
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch1[i+ii-4,j+jj-4]
                    ch1f[i,j]=sum

            ch2f=ch2
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch2[i+ii-4,j+jj-4]
                    ch2f[i,j]=sum
            print "ch2"
            ch3f=ch3
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch3[i+ii-4,j+jj-4]
                    ch3f[i,j]=sum
            print "ch3"
            ch4f=ch4
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch4[i+ii-4,j+jj-4]
                    ch4f[i,j]=sum
            print "ch4"
            ch5f=ch5
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch5[i+ii-4,j+jj-4]
                    ch5f[i,j]=sum
            print "ch5"
            ch6f=ch6
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch6[i+ii-4,j+jj-4]
                    ch6f[i,j]=sum
            print "ch6"
            ch7f=ch7
            for i in range(4,rr-4):
                for j in range(4,cc-4):
                    sum=0
                    for ii in range(0,9):
                        for jj in range(0,9):
                            sum=sum+ww[ii,jj]*ch7[i+ii-4,j+jj-4]
                    ch7f[i,j]=sum
            print "ch7"


        #print "Filtracja OK"
        arcpy.AddMessage("Filtration OK")


        #5. Obliczanie sumarycznego gradientu

        s1=Slope(ch1f,"PERCENT_RISE")
        s2=Slope(ch1f,"PERCENT_RISE")
        s3=Slope(ch1f,"PERCENT_RISE")
        s4=Slope(ch1f,"PERCENT_RISE")
        s5=Slope(ch1f,"PERCENT_RISE")
        s6=Slope(ch1f,"PERCENT_RISE")
        s7=Slope(ch1f,"PERCENT_RISE")
        ss=s1+s2+s3+s4+s5+s6+s7

        arcpy.AddMessage("Gradient calculations OK")

        # czyszczenie
        del s1;del s2;del s3;del s4;del s5;del s6;del s7
        del ch1f;del ch2f; del ch3f; del ch4f; del ch5f; del ch6f; del ch7f

        #6. tresholding and slope max


        chs=arcpy.RasterToNumPyArray(ss)
        ch1ff=chs*0.0
        for i in range(1,rr-1):
            for j in range(1,cc-1):
                if chs[i,j]<th:
                    ch1ff[i,j]=0.0
                else:
                    if (chs[i,j]>chs[i-1,j+1]) and (chs[i,j]>chs[i+1,j-1]):
                        ch1ff[i,j]=1.0
                    elif chs[i,j]>chs[i,j+1] and chs[i,j]>chs[i,j-1]:
                        ch1ff[i,j]=1.0
                    elif chs[i,j]>chs[i-1,j-1] and chs[i,j]>chs[i+1,j+1]:
                        ch1ff[i,j]=1.0
                    elif chs[i,j]>chs[i-1,j] and chs[i,j]>chs[i+1,j]:
                        ch1ff[i,j]=1.0
                    else:
                        ch1ff[i,j]=0.0


        for i in range(0,rr):
            for j in range(0,cc):
                if ch1ff[i,j]>0:
                    ch1ff[i,j]=1                

        for i in range(1,rr-1):
            for j in range(1,cc-1):
                if ch1ff[i,j]==1.0:
                    if ch1ff[i-1,j-1]==0 and ch1ff[i-1,j]==0 and ch1ff[i-1,j+1]==0 and ch1ff[i,j-1]==0 and ch1ff[i,j+1]==0 :
                        if  ch1ff[i+1,j-1]==0 and ch1ff[i+1,j]==0 and ch1ff[i+1,j+1]==0:
                            ch1ff[i,j]=0.0

        arcpy.AddMessage("Defining edges OK")


        # 7. Zapisywanie mapy krawedzi   mapedge
        ss=arcpy.NumPyArrayToRaster(ch1ff,point,cx,cy)
        ssi=Int(ss)

        if  edgestop ==1:
            arcpy.DefineProjection_management(ssi,sssref)
            ssi.save(mapedge)




        if edgestop ==0:
            # 8. Start tworzenia seedow
            
            seed=zeros((rr,cc),int)    # tablica stref-sedow
            idseed=1
            nn=15
            for inn in range(0,14):
               temp1=Con(ssi>0,ssi) #mapa dystansu od krawedzi
               seeddys= EucDistance(temp1)
               nn=15-inn
               pnn=int(nn/2)
               dpnn=pnn*cx
               temp2=Con(seeddys>dpnn,1,0) # maska dla seedow
               space=arcpy.RasterToNumPyArray(temp2)
               # iteracyjne tworzenie seedow
               for i in range(1,rr-nn):
                   for j in range(1,cc-nn):
                        if ch1ff[i,j]==0 and seed[i,j]==0 and space[i+pnn,j+pnn]>0: # not edge,not seed,enough space
                            pixedge=0           # test na edge
                            pixseed=0           # test na seed
                            for si in range(0,nn):
                                for sj in range(0,nn):
                                    if ch1ff[i+si,j+sj]==1:
                                        pixedge+=1
                                    if seed[i+si,j+sj]>0:
                                        pixseed+=1
            
                            if pixedge==0 and pixseed==0: # miejsce dla seeda OK
                                for si in range(0,nn): # tworzenie seeda
                                    for sj in range(0,nn):
                                        seed[i+si,j+sj]=idseed
                                idseed+=1

            #ssi=arcpy.NumPyArrayToRaster(seed,point,cx,cy)                     
            arcpy.AddMessage("Seeds OK")


            #9. korekta radiometryczna seedow, elementy seedow nie moga sie roznic o wiecej niz tau*std

            def NumpZonalStatisticsL(azone,avar,stat):
                ''' dziala jak ZonalStatistics ale dla numpy i bez ograniczenia liczby klas
                    azone - strefy, avar - zmienne, stat - rodzaj statystyki
                    zwraca tablice rozm. avar '''
                cc=azone.shape[1]
                rr=azone.shape[0]
                liczklas=amax(azone)
                zonstat1=zeros((liczklas+1),float)
                zonstat2=zeros((liczklas+1),float)
                if stat=="MEAN":  
                    for i in range(1,rr):
                        for j in range(1,cc):
                            klas=int(azone[i,j])
                            zonstat1[klas]+=avar[i,j]
                            zonstat2[klas]+=1
                    zonstat1[0]=0.0                    
                    for i in range(1,liczklas):
                        if zonstat2[i]>0:
                             zonstat1[i]=zonstat1[i]/zonstat2[i]
                    temp=avar*0.0
                    for i in range(1,rr):
                        for j in range(1,cc):
                            klas=int(azone[i,j])
                            temp[i,j]=zonstat1[klas]
                    return (temp)

            
            # obliczenie std
            std1=l8ch1.standardDeviation
            std2=l8ch2.standardDeviation
            std3=l8ch3.standardDeviation
            std4=l8ch4.standardDeviation
            std5=l8ch5.standardDeviation
            std6=l8ch6.standardDeviation
            std7=l8ch7.standardDeviation
            cellsize=l8ch1.meanCellWidth


            
            ch1zm=NumpZonalStatisticsL(seed,ch1,"MEAN")
            ch2zm=NumpZonalStatisticsL(seed,ch2,"MEAN")
            ch3zm=NumpZonalStatisticsL(seed,ch3,"MEAN")
            ch4zm=NumpZonalStatisticsL(seed,ch4,"MEAN")
            ch5zm=NumpZonalStatisticsL(seed,ch5,"MEAN")
            ch6zm=NumpZonalStatisticsL(seed,ch6,"MEAN")
            ch7zm=NumpZonalStatisticsL(seed,ch7,"MEAN")
            # warunek podobienstwa radiometrycznego
            for i in range(0,rr):
               for j in range(0,cc):
                    if seed[i,j]>0:
                        rad=0
                        rad=ww1*(((ch1[i,j]-ch1zm[i,j])**2)/(tau*std1)**2)+rad
                        rad=ww2*(((ch2[i,j]-ch2zm[i,j])**2)/(tau*std2)**2)+rad
                        rad=ww3*(((ch3[i,j]-ch3zm[i,j])**2)/(tau*std3)**2)+rad
                        rad=ww4*(((ch4[i,j]-ch4zm[i,j])**2)/(tau*std4)**2)+rad
                        rad=ww5*(((ch5[i,j]-ch5zm[i,j])**2)/(tau*std5)**2)+rad
                        rad=ww6*(((ch6[i,j]-ch6zm[i,j])**2)/(tau*std6)**2)+rad
                        rad=ww7*(((ch7[i,j]-ch7zm[i,j])**2)/(tau*std7)**2)+rad
                        if rad>1.0:
                            seed[i,j]=0

            arcpy.AddMessage("Radiometric correction of seeds OK")

            #ssi=arcpy.NumPyArrayToRaster(seed,point,cx,cy)
            del ch1zm;del ch2zm;del ch3zm;del ch4zm;del ch5zm;del ch6zm;del ch7zm
            

            
            # 10. segment growing  iteracja

            iter=0
            lll=10000000
            while lll>0 and iter <30: #zaklada sie 30 iteracji
                if lll<50:
                   tau=1.5
                if lll<50 and iter>20:
                   tau=2.0
                if iter>30:
                    tau=2.5

                # dla nowych stref
                ch1zm=NumpZonalStatisticsL(seed,ch1,"MEAN")
                ch2zm=NumpZonalStatisticsL(seed,ch2,"MEAN")
                ch3zm=NumpZonalStatisticsL(seed,ch3,"MEAN")
                ch4zm=NumpZonalStatisticsL(seed,ch4,"MEAN")
                ch5zm=NumpZonalStatisticsL(seed,ch5,"MEAN")
                ch6zm=NumpZonalStatisticsL(seed,ch6,"MEAN")
                ch7zm=NumpZonalStatisticsL(seed,ch7,"MEAN")

                env.workspace=wspace    
                rseed=arcpy.NumPyArrayToRaster(seed,point,cx,cy)
                arcpy.DefineProjection_management(rseed,sssref)
                seednd=Con(rseed>0,rseed)
                seedalloc = EucAllocation(seednd,cellsize)
                seedalloc2=Con(IsNull(seedalloc),0,seedalloc)
                temp=seedalloc2-rseed
                rkand=Con(temp>0,1,0)
                kand=arcpy.RasterToNumPyArray(rkand)
                del seednd;del seedalloc;del seedalloc2;del rkand
                lll=0
                rad1,rad2,rad3,rad4=1.0,1.0,1.0,1.0
                for i in range(1,rr-1):
                   for j in range(1,cc-1):
                        if kand[i,j]==1:
                            idk1=0;idk2=0;idk3=0;idk4=0
                            nklas=0
                            if seed[i-1,j]>0:
                                idk1=0   
                                rad=0
                                
                                rad=ww1*(((ch1[i,j]-ch1zm[i-1,j])**2)/(tau*std1)**2)+rad
                                rad=ww2*(((ch2[i,j]-ch2zm[i-1,j])**2)/(tau*std2)**2)+rad
                                rad=ww3*(((ch3[i,j]-ch3zm[i-1,j])**2)/(tau*std3)**2)+rad
                                rad=ww4*(((ch7[i,j]-ch7zm[i-1,j])**2)/(tau*std7)**2)+rad
                                rad=ww5*(((ch4[i,j]-ch4zm[i-1,j])**2)/(tau*std4)**2)+rad
                                rad=ww6*(((ch5[i,j]-ch5zm[i-1,j])**2)/(tau*std5)**2)+rad
                                rad=ww7*(((ch6[i,j]-ch6zm[i-1,j])**2)/(tau*std6)**2)+rad

                                if rad<=1.0:
                                    idk1=seed[i-1,j]
                                    rad1=rad
                                    nklas+=1
                            if seed[i+1,j]>0:
                                idk2=0    
                                rad=0
                                
                                rad=ww1*(((ch1[i,j]-ch1zm[i+1,j])**2)/(tau*std1)**2)+rad
                                rad=ww2*(((ch2[i,j]-ch2zm[i+1,j])**2)/(tau*std2)**2)+rad
                                rad=ww3*(((ch3[i,j]-ch3zm[i+1,j])**2)/(tau*std3)**2)+rad
                                rad=ww4*(((ch7[i,j]-ch7zm[i+1,j])**2)/(tau*std7)**2)+rad
                                rad=ww5*(((ch4[i,j]-ch4zm[i+1,j])**2)/(tau*std4)**2)+rad
                                rad=ww6*(((ch5[i,j]-ch5zm[i+1,j])**2)/(tau*std5)**2)+rad
                                rad=ww7*(((ch6[i,j]-ch6zm[i+1,j])**2)/(tau*std6)**2)+rad

                                if rad<=1.0:
                                    idk2=seed[i+1,j]
                                    rad2=rad
                                    nklas+=1
                            if seed[i,j+1]>0:
                                idk3=0    
                                rad=0
                              
                                rad=ww1*(((ch1[i,j]-ch1zm[i,j+1])**2)/(tau*std1)**2)+rad
                                rad=ww2*(((ch2[i,j]-ch2zm[i,j+1])**2)/(tau*std2)**2)+rad
                                rad=ww3*(((ch3[i,j]-ch3zm[i,j+1])**2)/(tau*std3)**2)+rad
                                rad=ww4*(((ch7[i,j]-ch7zm[i,j+1])**2)/(tau*std7)**2)+rad
                                rad=ww5*(((ch4[i,j]-ch4zm[i,j+1])**2)/(tau*std4)**2)+rad
                                rad=ww6*(((ch5[i,j]-ch5zm[i,j+1])**2)/(tau*std5)**2)+rad
                                rad=ww7*(((ch6[i,j]-ch6zm[i,j+1])**2)/(tau*std6)**2)+rad
                                if rad<=1.0:
                                    idk3=seed[i,j+1]
                                    rad3=rad
                                    nklas+=1
                            if seed[i,j-1]>0:
                                idk4=0    
                                rad=0
                             
                                rad=ww1*(((ch1[i,j]-ch1zm[i,j-1])**2)/(tau*std1)**2)+rad
                                rad=ww2*(((ch2[i,j]-ch2zm[i,j-1])**2)/(tau*std2)**2)+rad
                                rad=ww3*(((ch3[i,j]-ch3zm[i,j-1])**2)/(tau*std3)**2)+rad
                                rad=ww4*(((ch7[i,j]-ch7zm[i,j-1])**2)/(tau*std7)**2)+rad
                                rad=ww5*(((ch4[i,j]-ch4zm[i,j-1])**2)/(tau*std4)**2)+rad
                                rad=ww6*(((ch5[i,j]-ch5zm[i,j-1])**2)/(tau*std5)**2)+rad
                                rad=ww7*(((ch6[i,j]-ch6zm[i,j-1])**2)/(tau*std6)**2)+rad

                                if rad<=1.0:
                                    idk4=seed[i,j-1]
                                    rad4=rad
                                    nklas+=1
                            idk=idk1+idk2+idk3+idk4
                            if idk>0:
                                if nklas==1:
                                    if idk1>0:
                                        seed[i,j]=idk1
                                        lll+=1
                                    elif idk2>0:
                                        seed[i,j]=idk2
                                        lll+=1
                                    elif idk3>0:
                                        seed[i,j]=idk3
                                        lll+=1
                                    elif idk4>0:
                                        seed[i,j]=idk4
                                        lll+=1
                                else:
                                    if idk1>0 and idk2>0:
                                        if rad1<rad2:
                                            seed[i,j]=idk1
                                            lll+=1
                                        else:
                                            seed[i,j]=idk2
                                            lll+=1
                                    elif idk1>0 and idk3>0:
                                        if rad1<rad3:
                                            seed[i,j]=idk1
                                            lll+=1
                                        else:
                                            seed[i,j]=idk3
                                            lll+=1
                                    elif idk1>0 and idk4>0:
                                        if rad1<rad3:
                                            seed[i,j]=idk1
                                            lll+=1
                                        else:
                                            seed[i,j]=idk4
                                            lll+=1
                                    elif idk2>0 and idk3>0:
                                        if rad2<rad3:
                                            seed[i,j]=idk2
                                            lll+=1
                                        else:
                                            seed[i,j]=idk3
                                            lll+=1
                                    elif idk3>0 and idk4>0:
                                        if rad3<rad4:
                                            seed[i,j]=idk3
                                            lll+=1
                                        else:
                                            seed[i,j]=idk4
                                            lll+=1



                del ch1zm;del ch2zm;del ch3zm;del ch4zm;del ch5zm;del ch6zm;del ch7zm
                ss2= "iteration "+str(iter)+" sd "+str(tau)+" pixels "+str(lll)
                arcpy.AddMessage(ss2)

                iter+=1
                del kand


            # 11. utworzenie rastra wynikowego z nadaniem georeferencji
            ss=arcpy.NumPyArrayToRaster(seed,point,cx,cy)
            ssi3=Int(ss)
            arcpy.DefineProjection_management(ssi3,sssref)
            #wypelnienie pustych za pomoca alokacji
            ssi2=Con(ssi3>0,ssi3)
            rseed = EucAllocation(ssi2)
            #rseed2.save(mapklas)
            arcpy.AddMessage("Growing seed OK")
            #print "Growing seed OK"

            # agregacja klas
           
            seed=arcpy.RasterToNumPyArray(rseed)

            lklas=amax(seed)
            ltab=zeros((lklas+1,7),float)
            
            def NumpZonalStatisticsLTAB(azone,avar,stat,nn,ltab):   #wypelnia tablice ltab
                # liczba klas
                cc=azone.shape[1]
                rr=azone.shape[0]
                liczklas=amax(azone)
                #global ltab
                zonstat2=zeros((liczklas+1),float)
                if stat=="MEAN":  
                    for i in range(0,rr):
                        for j in range(0,cc):
                            klas=int(azone[i,j])
                            ltab[klas,nn]=ltab[klas,nn]+avar[i,j]
                            zonstat2[klas]=zonstat2[klas]+1
                    ltab[0,nn]=0.0                    
                    for i in range(1,liczklas):
                        if zonstat2[i]>0:
                             ltab[i,nn]=ltab[i,nn]/zonstat2[i]

            NumpZonalStatisticsLTAB(seed,ch1,"MEAN",0,ltab)

            NumpZonalStatisticsLTAB(seed,ch2,"MEAN",1,ltab)

            NumpZonalStatisticsLTAB(seed,ch3,"MEAN",2,ltab)

            NumpZonalStatisticsLTAB(seed,ch4,"MEAN",3,ltab)

            NumpZonalStatisticsLTAB(seed,ch5,"MEAN",4,ltab)

            NumpZonalStatisticsLTAB(seed,ch6,"MEAN",5,ltab)

            NumpZonalStatisticsLTAB(seed,ch7,"MEAN",6,ltab)

            def testagreg(ob1,ob2,s1,s2,s3,s4,s5,s6,s7,w1,w2,w3,w4,w5,w6,w7,ltab):
                
                tau2=0.5
                rad=0
                rad=w1*(((ltab[ob1,0]-ltab[ob2,0])**2)/(tau2*s1)**2)+rad
                rad=w2*(((ltab[ob1,1]-ltab[ob2,1])**2)/(tau2*s2)**2)+rad
                rad=w3*(((ltab[ob1,2]-ltab[ob2,2])**2)/(tau2*s3)**2)+rad
                rad=w4*(((ltab[ob1,3]-ltab[ob2,3])**2)/(tau2*s4)**2)+rad
                rad=w5*(((ltab[ob1,4]-ltab[ob2,4])**2)/(tau2*s5)**2)+rad
                rad=w6*(((ltab[ob1,5]-ltab[ob2,5])**2)/(tau2*s6)**2)+rad
                rad=w7*(((ltab[ob1,6]-ltab[ob2,6])**2)/(tau2*s7)**2)+rad

                if rad<=1.0:
                    return 1
                else:
                    return 0

            nowa=zeros((lklas+1),int)
            slo1=Slope(rseed,"PERCENT_RISE")
            slo2=Con(slo1>0,1,0)
            nslo2=arcpy.RasterToNumPyArray(slo2)
            lllnowe=1

            for i in range(1,rr-1):
                  for j in range(1,cc-1):
                    if nslo2[i,j]>0:
                        kan=1
                        if seed[i,j]!=seed[i,j+1]:
                            kan1=seed[i,j]
                            kan2=seed[i,j+1]
                        elif seed[i,j]!=seed[i+1,j]:
                            kan1=seed[i,j]
                            kan2=seed[i+1,j]
                        elif seed[i,j]!=seed[i,j-1]:
                            kan1=seed[i,j]
                            kan2=seed[i,j-1]
                        elif seed[i,j]!=seed[i-1,j]:
                            kan1=seed[i,j]
                            kan2=seed[i-1,j]
                        else:
                            kan=0
                        if kan==1:
                            if nowa[kan1]==0 and nowa[kan2]==0:
                                k=testagreg(kan1,kan2,std1,std2,std3,std4,std5,std6,std7,ww1,ww2,ww3,ww4,ww5,ww6,ww7,ltab)
                                if k==1:
                                    nowa[kan1]=lllnowe
                                    nowa[kan2]=lllnowe
                                    lllnowe+=1
                                else:
                                    nowa[kan1]=lllnowe
                                    lllnowe+=1
                                    nowa[kan2]=lllnowe
                                    lllnowe+=1 
                            elif nowa[kan1]>0 and nowa[kan2]==0:
                                k=testagreg(kan1,kan2,std1,std2,std3,std4,std5,std6,std7,ww1,ww2,ww3,ww4,ww5,ww6,ww7,ltab)
                                if k==1:
                                    nowa[kan2]=nowa[kan1]
                                else:
                                    nowa[kan2]=lllnowe
                                    lllnowe+=1
                            elif nowa[kan1]==0 and nowa[kan2]>0:
                                k=testagreg(kan1,kan2,std1,std2,std3,std4,std5,std6,std7,ww1,ww2,ww3,ww4,ww5,ww6,ww7,ltab)
                                if k==1:
                                    nowa[kan1]=nowa[kan2]
                                else:
                                    nowa[kan1]=lllnowe
                                    lllnowe+=1

            arcpy.AddMessage("Agregation OK")

            for i in range(1,lklas+1):
                if nowa[i]==0:
                    nowa[i]=lllnowe
                    lllnowe+=1
            seedgen=seed
            for i in range(1,rr-1):
                for j in range(1,cc-1):
                    kk=int(seed[i,j])
                    seedgen[i,j]=nowa[kk]
                                 
                            
            ss22=arcpy.NumPyArrayToRaster(seedgen,point,cx,cy)
            rseed=RegionGroup(ss22) # kazda grupa oddzielnie

            seed=arcpy.RasterToNumPyArray(rseed)
            cc=seed.shape[1]
            rr=seed.shape[0]



            def SeedClean(azone):
                    # liczba klas
                    cc=azone.shape[1]
                    rr=azone.shape[0]
                    liczklas=amax(azone)
                    zonstat2=zeros((liczklas+1),int)
                    for i in range(1,rr):
                        for j in range(1,cc):
                                klas=int(azone[i,j])
                                zonstat2[klas]+=1

                    temp=seed*0.0
                    for i in range(1,rr):
                        for j in range(1,cc):
                                klas=int(azone[i,j])
                                temp[i,j]=zonstat2[klas]
                   
                    return (temp)



            minpow=10
            temp1=SeedClean(seed)
            ss2=arcpy.NumPyArrayToRaster(temp1,point,cx,cy)
            arcpy.DefineProjection_management(ss2,sssref)
            ss3=Con(ss2>=minpow,rseed)
            ss3i=Int(ss3)
            ss4a=RegionGroup(ss3i)
            ss4=EucAllocation(ss4a)
            arcpy.RasterToPolygon_conversion(ss4, mapklas,"","VALUE")
            mapklas2=mapklas[:len(mapklas)-4]
            mapklas2=mapklas2+".dbf"
            arcpy.AddField_management(mapklas2,"OBID", "LONG")
            arcpy.CalculateField_management(mapklas2,"OBID", "!ID!", "PYTHON_9.3")
            arcpy.DeleteField_management(mapklas2,["ID","GRIDCODE"])

              
            
            
            
