## Tworzy kompozyt z pansharpeningiem z okna .TIF
## Opcjonalnie zapisuje wyciete kanaly z roz. 30m albo 7.5m
## Dla wycietych kanalow tworzy plik z ich nazwami i parametrami z MTL.txt
## Tool do pracy w katalogu po rozpakowaniu zdjecia Landsat8

import arcpy
from arcpy import env
from arcpy.sa import*
from numpy import*
import os

class CompositPansharp(object):
    """Tworzy kompozyt z pansharpeningiem i zapisuje kanaly o roz. 30 lub 15 m."""
    def __init__(self):
        self.label = "Pan-sharpened composit"
        self.description =  ("Create pan-sharpened composit for defined rectengular window and save channels with 30m or 15m resolution.")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
        # Input parameter
        dirr = arcpy.Parameter(
            displayName="Workspace",
            name="dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_mtl = arcpy.Parameter(
            displayName="Meta data Landsat 8 MTL text file",
            name="in_mtl",
            datatype="DETextfile",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_aoi = arcpy.Parameter(
            displayName="Area of interest rectangle",
            name="in_aoi",
            datatype="DEShapefile",
            parameterType="Required",
            direction="Input")
         # Input parameter
        in_ppp = arcpy.Parameter(
            displayName="Project name",
            name="in_ppp",
            datatype="GPString",
            parameterType="Required",
            direction="Input")
        # Input parameter
        r30 = arcpy.Parameter(
            displayName="Resolution 30m - without pansharpening ",
            name="r30",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        r30.value=False
         # Input parameter
        r75 = arcpy.Parameter(
            displayName="Resolution 15m",
            name="r75",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        r75.value=True
        # Input parameter
        rff = arcpy.Parameter(
            displayName="Smoothing filter only for 15m",
            name="rff",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        rff.value=False
        
        
        parameters=[dirr,in_mtl,in_aoi,in_ppp,r30,r75,rff]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):

##
##        # zmienne
##        wspace  katalog roboczy      string
##        ssMTL   plik MTL             string
##        obrys   aoi okno do wyciecia     string
##        kod30   opcja zapisu 30      integer  (0/1)
##        kod15   opcja zapisu 15     integer  (0/1)
##        kodff   opcja zapisu filtracji integer (0/1)

        wspace=parameters[0].valueAsText
        ssMTL=parameters[1].valueAsText
        obrys=parameters[2].valueAsText
        csssa=parameters[3].valueAsText
        bkod30=parameters[4].value
        bkod15=parameters[5].value
        bkodff=parameters[6].value


        if bkod15==1:
            kod15=1
        else:
            kod15=0
        if bkodff==1:
            kodff=1
        else:
            kodff=0
        if bkod30==0:  # brak realizacji 15 i filtracji
            kod30=0
        else:
            kod30=1
            kod15=0
            kodff=0
        if kodff==1 and kod15==1: # tylko dla 15
            kodff=1
        else:
            kodff=0

        # test warunkow
        ss2="k30 "+str(kod30)+"  "+"k15 "+str(kod15)+"  "+"kff "+str(kodff)
        arcpy.AddMessage(ss2)
            

        
        env.workspace=wspace

        # okreslenie zmiennych z pliku MTL

        L8date = ""   # data
        L8time = ""   # godzina
        L8sunelev=0.0 # wys. slonca w stopniach
        L8sundistance=0.0  # zmiana odleglosci do slonca 1.0 srednia
        L8name=["L8"]*12   # 12 kanalow - 1,2,3,4,5,6,7,8,9,10,11,maska
        L8rad=zeros((11,2),float) # 11 kanalow do obliczenia radiance  ML, AL
        L8ref=zeros((9,2),float)  #  9 kanslow do obliczenia reflectance MR, AR
        L8K=zeros((2,2),float)    # kanaly 10 i 11 do obliczenia brightnes temp K1,K2

       # czytanie pliku MTL

        nne=0
        irad=0
        irad2=0
        nnra1=0
        nnra2=0
        nnrb1=0
        nnrb2=0
        nnrc1=0
        nnrc2=0
        in_plik=open(ssMTL,"r")

        for line in in_plik.readlines():
            lista=line.split(' ')
            #print "X"
            if len(lista) >4 and lista[4]=='DATE_ACQUIRED': #L8date
                L8date=lista[6]
                L8date=L8date[0:10]
            if len(lista) >4 and lista[4]=='SCENE_CENTER_TIME':  #L8time      
                L8time=lista[6]
                L8time=L8time[0:5]+" GMT"
            if len(lista) >4 and lista[4]=='SUN_ELEVATION':   #L8sunelev      
                L8sunelev=float(lista[6])
            if len(lista) >4 and lista[4]=='EARTH_SUN_DISTANCE': #L8sundistance       
                L8sundistance=float(lista[6])
            if len(lista) >4 and lista[4]=='FILE_NAME_BAND_1':  #L8name 1     
                L8=lista[6]
                nn=len(L8)
                L8name[nne]=L8[1:nn-2]
                nne+=1
            if nne>0 and nne< 12 and lista[4]<>'FILE_NAME_BAND_1': #L8name 2-11
                L8=lista[6]
                nn=len(L8)
                L8name[nne]=L8[1:nn-2]
                nne+=1
            if irad>0 and irad< 41: # read rad and ref rescaling coeff
                if irad<12:
                    L8rad[nnra1,0]=float(lista[6])
                    nnra1+=1
                if irad>=12 and irad<23:
                    L8rad[nnra2,1]=float(lista[6])
                    nnra2+=1
                if irad>=23 and irad<32:
                    L8ref[nnrb1,0]=float(lista[6])
                    nnrb1+=1
                if irad>=32 and irad<41:
                    L8ref[nnrb2,1]=float(lista[6])
                    nnrb2+=1
                irad+=1    
            if irad2>0 and irad2<5: # read thermal constants
                if irad2<3:
                    L8K[nnrc1,0]=float(lista[6])
                    nnrc1+=1
                if irad2>=3:
                    L8K[nnrc2,1]=float(lista[6])
                    nnrc2+=1
                irad2+=1    
            if len(lista) >3 and lista[4]=='RADIOMETRIC_RESCALING\n' and irad==0:  # start reading radiometric
                irad=1;
            if len(lista) >3 and lista[4]=='TIRS_THERMAL_CONSTANTS\n' and irad2==0:  # start reading termal co.
                irad2=1;


        in_plik.close()        



        # okreslenie geometrii obrysu

        des=arcpy.Describe(obrys)
        ext=des.Extent
        xmin=ext.XMin;ymin=ext.YMin;xmax=ext.XMax;ymax=ext.YMax
        envp=str(xmin)+" "+str(ymin)+" "+str(xmax)+" "+str(ymax)
        #arcpy.AddMessage(envp)
        #arcpy.CheckOutExtension("Spatial")
        env.overwriteOutput=True
        
        # wczytanie zdjec
        l8ch8=Raster(L8name[7])
        l8ch1=Raster(L8name[0])
        l8ch2=Raster(L8name[1])
        l8ch3=Raster(L8name[2])
        l8ch4=Raster(L8name[3])
        l8ch5=Raster(L8name[4])
        l8ch6=Raster(L8name[5])
        l8ch7=Raster(L8name[6])
        

        # korekcja wartosci ujemnych
        c1=Con(l8ch1>0,l8ch1)
        c2=Con(l8ch2>0,l8ch2)
        c3=Con(l8ch3>0,l8ch3)
        c4=Con(l8ch4>0,l8ch4)
        c5=Con(l8ch5>0,l8ch5)
        c6=Con(l8ch6>0,l8ch6)
        c7=Con(l8ch7>0,l8ch7)


        # obliczenie minimum sygnalu i   
        mint1=c1.minimum
        mint2=c2.minimum
        mint3=c3.minimum
        mint4=c4.minimum
        mint5=c5.minimum
        mint6=c6.minimum
        mint7=c7.minimum


        rmint1=mint1*L8rad[0,0]+L8rad[0,1]
        rmint2=mint2*L8rad[1,0]+L8rad[0,1]
        rmint3=mint3*L8rad[2,0]+L8rad[0,1]
        rmint4=mint4*L8rad[3,0]+L8rad[0,1]
        rmint5=mint5*L8rad[4,0]+L8rad[0,1]
        rmint6=mint6*L8rad[5,0]+L8rad[0,1]
        rmint7=mint7*L8rad[6,0]+L8rad[0,1]

        del c1;del c2;del c3;del c4;del c5;del c6;del c7

        # wycinanie rastrow do aoi i ich zapisywanie 30
        if kod15==1:
            csss="L8ch8_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch8,"#",csss,obrys, "#", "ClippingGeometry")
            csss1="L8ch1_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch1,"#",csss1,obrys, "#", "ClippingGeometry")
            csss2="L8ch2_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch2,"#",csss2,obrys, "#", "ClippingGeometry")
            csss3="L8ch3_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch3,"#",csss3,obrys, "#", "ClippingGeometry")
            csss4="L8ch4_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch4,"#",csss4,obrys, "#", "ClippingGeometry")
            csss5="L8ch5_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch5,"#",csss5,obrys, "#", "ClippingGeometry")
            csss6="L8ch6_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch6,"#",csss6,obrys, "#", "ClippingGeometry")
            csss7="L8ch7_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch7,"#",csss7,obrys, "#", "ClippingGeometry")
        if kod30==1:
            csss1="L8ch1_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch1,"#",csss1,obrys, "#", "ClippingGeometry")
            csss2="L8ch2_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch2,"#",csss2,obrys, "#", "ClippingGeometry")
            csss3="L8ch3_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch3,"#",csss3,obrys, "#", "ClippingGeometry")
            csss4="L8ch4_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch4,"#",csss4,obrys, "#", "ClippingGeometry")
            csss5="L8ch5_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch5,"#",csss5,obrys, "#", "ClippingGeometry")
            csss6="L8ch6_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch6,"#",csss6,obrys, "#", "ClippingGeometry")
            csss7="L8ch7_"+L8date+"_"+csssa+".tif"
            arcpy.Clip_management(l8ch7,"#",csss7,obrys, "#", "ClippingGeometry")
            arcpy.env.cellSize=30
            c432=csss1+";"+csss2+";"+csss3+";"+csss4+";"+csss5+";"+csss6+";"+csss7
            csss="L8comp_"+L8date+"_"+csssa+".tif"
            arcpy.CompositeBands_management(c432,csss)
            
        del l8ch8;del l8ch1;del l8ch2;del l8ch3;del l8ch4;del l8ch5;del l8ch6;del l8ch7

        if kod15==1:
            # resampling ch8 do 15m
            csss="L8ch8_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management(csss,"ch8r.tif","7.5","NEAREST")

            pan8=Raster("ch8r.tif")
            csss="L8ch1_"+L8date+"_"+csssa+".tif"
            c1=Raster(csss)
            csss="L8ch2_"+L8date+"_"+csssa+".tif"
            c2=Raster(csss)
            csss="L8ch3_"+L8date+"_"+csssa+".tif"
            c3=Raster(csss)
            csss="L8ch4_"+L8date+"_"+csssa+".tif"
            c4=Raster(csss)
            csss="L8ch5_"+L8date+"_"+csssa+".tif"
            c5=Raster(csss)
            csss="L8ch6_"+L8date+"_"+csssa+".tif"
            c6=Raster(csss)
            csss="L8ch7_"+L8date+"_"+csssa+".tif"
            c7=Raster(csss)

            #okreslenie offsetu obu rastrow
            x30=c2.extent.XMin
            y30=c2.extent.YMax
            x75=pan8.extent.XMin
            y75=pan8.extent.YMax
            opx= int((x30-x75)/7.5)
            opy=int((y75-y30)/7.5)

            #okreslenie georeferencji
            sssref=pan8.spatialReference.exporttostring()
            cx=pan8.meanCellWidth
            cy=pan8.meanCellHeight
            licx=pan8.extent.XMin
            licy=pan8.extent.YMin
            point=arcpy.CreateObject("point")
            point.X=licx
            point.Y=licy

            n_pan8=arcpy.RasterToNumPyArray(pan8)

            cc=n_pan8.shape[1]
            rr=n_pan8.shape[0]


            n_pan8sum=ones((rr,cc),float)

            rr4=rr/4
            cc4=cc/4

            ot=1   #offset aby kom 4x4 7.5 pokrywala kom 30
            for ii in range(1,rr4-1):
                for jj in range(1,cc4-1):
                    suma=0
                    for i in range(ii*4+opy,ii*4+4+opy):
                        for j in range(jj*4+opx,jj*4+4+opx):
                            suma=suma+n_pan8[i,j]
                    smean=suma/16.0        
                    for i in range(ii*4+opy,ii*4+4+opy):
                        for j in range(jj*4+opx,jj*4+4+opx):
                            n_pan8sum[i,j]=n_pan8[i,j]/smean
                            #n_pan8sum[i,j]=smean

            pan8sum=arcpy.NumPyArrayToRaster(n_pan8sum,point,cx,cy)
            arcpy.DefineProjection_management(pan8sum,sssref)


            arcpy.env.cellSize=7.5
            ch1x=Int(pan8sum*c1)
            ch2x=Int(pan8sum*c2)
            ch3x=Int(pan8sum*c3)
            ch4x=Int(pan8sum*c4)
            ch5x=Int(pan8sum*c5)
            ch6x=Int(pan8sum*c6)
            ch7x=Int(pan8sum*c7)
            del c1;del c2;del c3;del c4;del c5;del c6;del c7
            del pan8sum

            if kodff==1:
                ch1xf =  Filter(ch1x, "LOW", "")
                ch2xf =  Filter(ch2x, "LOW", "")
                ch3xf =  Filter(ch3x, "LOW", "")
                ch4xf =  Filter(ch4x, "LOW", "")
                ch5xf =  Filter(ch5x, "LOW", "")
                ch6xf =  Filter(ch6x, "LOW", "")
                ch7xf =  Filter(ch7x, "LOW", "")
                ch1xf.save("ch1xx.tif")
                ch2xf.save("ch2xx.tif")
                ch3xf.save("ch3xx.tif")
                ch4xf.save("ch4xx.tif")
                ch5xf.save("ch5xx.tif")
                ch6xf.save("ch6xx.tif")
                ch7xf.save("ch7xx.tif")
                del ch1xf;del ch2xf;del ch3xf;del ch4xf;del ch5xf;del ch6xf;del ch7xf
            else:  
                ch1x.save("ch1xx.tif")
                ch2x.save("ch2xx.tif")
                ch3x.save("ch3xx.tif")
                ch4x.save("ch4xx.tif")
                ch5x.save("ch5xx.tif")
                ch6x.save("ch6xx.tif")
                ch7x.save("ch7xx.tif")
            del ch1x;del ch2x;del ch3x;del ch4x;del ch5x;del ch6x;del ch7x            
            csss1="L8ch1x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch1xx.tif", csss1, "15", "CUBIC")
            csss2="L8ch2x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch2xx.tif", csss2, "15", "CUBIC")
            csss3="L8ch3x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch3xx.tif", csss3, "15", "CUBIC")
            csss4="L8ch4x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch4xx.tif", csss4, "15", "CUBIC")
            csss5="L8ch5x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch5xx.tif", csss5, "15", "CUBIC")
            csss6="L8ch6x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch6xx.tif", csss6, "15", "CUBIC")
            csss7="L8ch7x_"+L8date+"_"+csssa+".tif"
            arcpy.Resample_management("ch7xx.tif", csss7, "15", "CUBIC")
           
            arcpy.env.cellSize=15
            c432=csss1+";"+csss2+";"+csss3+";"+csss4+";"+csss5+";"+csss6+";"+csss7
         
            csss="L8comp_"+L8date+"_"+csssa+".tif"
            arcpy.CompositeBands_management(c432,csss)

            csss="L8infox_"+L8date+"_"+csssa+".txt"
            ssstxt=wspace+"\\"+csss
            out_plik=open(ssstxt,"w")
            out_plik.write("L8DN15m"+"\n")
            out_plik.write("Nazwy plikow"+"\n")
            csss="L8ch1x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch2x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch3x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch4x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch5x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch6x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch7x_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            out_plik.write("wsp do oblicz radiancji"+"\n")
            for i in range(0,11):
                out_plik.write(str(L8rad[i,0])+","+str(L8rad[i,1])+"\n")
            out_plik.write("wsp do oblicz reflektancji"+"\n")
            for i in range(0,9):
                out_plik.write(str(L8ref[i,0])+","+str(L8ref[i,1])+"\n")
            out_plik.write("Parametry"+"\n")
            out_plik.write(L8date+","+"data"+"\n")
            out_plik.write(L8time+","+"godzina"+"\n")
            out_plik.write(str(L8sunelev)+","+"wys slonca"+"\n")
            out_plik.write(str(L8sundistance)+","+"odleglosc od slonca"+"\n")
            out_plik.write("Minimum TOA radiance in ch 1,2,3,4,5,6,7"+"\n")
            out_plik.write(str(rmint1)+","+str(mint1)+"\n")
            out_plik.write(str(rmint2)+","+str(mint2)+"\n")
            out_plik.write(str(rmint3)+","+str(mint3)+"\n")
            out_plik.write(str(rmint4)+","+str(mint4)+"\n")
            out_plik.write(str(rmint5)+","+str(mint5)+"\n")
            out_plik.write(str(rmint6)+","+str(mint6)+"\n")
            out_plik.write(str(rmint7)+","+str(mint7)+"\n")
            out_plik.close()

        if kod30==1:   
            csss="L8info_"+L8date+"_"+csssa+".txt"
            ssstxt=wspace+"\\"+csss
            out_plik=open(ssstxt,"w")
            out_plik.write("L8DN30m"+"\n")
            out_plik.write("Nazwy plikow"+"\n")
            csss="L8ch1_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch2_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch3_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch4_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch5_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch6_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            csss="L8ch7_"+L8date+"_"+csssa+".tif"
            out_plik.write(csss+"\n")
            out_plik.write("wsp do oblicz radiancji"+"\n")
            for i in range(0,11):
                out_plik.write(str(L8rad[i,0])+","+str(L8rad[i,1])+"\n")
            out_plik.write("wsp do oblicz reflektancji"+"\n")
            for i in range(0,9):
                out_plik.write(str(L8ref[i,0])+","+str(L8ref[i,1])+"\n")
            out_plik.write("Parametry"+"\n")
            out_plik.write(L8date+","+"data"+"\n")
            out_plik.write(L8time+","+"godzina"+"\n")
            out_plik.write(str(L8sunelev)+","+"wys slonca"+"\n")
            out_plik.write(str(L8sundistance)+","+"odleglosc od slonca"+"\n")
            out_plik.write("Minimum TOA radiance in ch 1,2,3,4,5,6,7"+"\n")
            out_plik.write(str(rmint1)+","+str(mint1)+"\n")
            out_plik.write(str(rmint2)+","+str(mint2)+"\n")
            out_plik.write(str(rmint3)+","+str(mint3)+"\n")
            out_plik.write(str(rmint4)+","+str(mint4)+"\n")
            out_plik.write(str(rmint5)+","+str(mint5)+"\n")
            out_plik.write(str(rmint6)+","+str(mint6)+"\n")
            out_plik.write(str(rmint7)+","+str(mint7)+"\n")
            out_plik.close()
                
        arcpy.Delete_management("ch8r.tif")
        arcpy.Delete_management("ch1xx.tif")
        arcpy.Delete_management("ch2xx.tif")
        arcpy.Delete_management("ch3xx.tif")
        arcpy.Delete_management("ch4xx.tif")
        arcpy.Delete_management("ch5xx.tif")
        arcpy.Delete_management("ch6xx.tif")
        arcpy.Delete_management("ch7xx.tif")

        #arcpy.CheckInExtension("Spatial")
