import arcpy
import math
from arcpy import env
from arcpy.sa import*
from numpy import*

class RadianceAtmcor(object):
    """Oblicza radiancje z korekcja atmosferyczna."""
    def __init__(self):
        self.label = "Radiance with atmospheric correction"
        self.description =  ("Calculate radiance with DOS 1% atmospheric correction")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
        # Input parameter
        dirr = arcpy.Parameter(
            displayName="Workspace",
            name="dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_mtx = arcpy.Parameter(
            displayName="Meta data text file (from Pan-sharpened composit tool)",
            name="in_mtx",
            datatype="DETextfile",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_ppp = arcpy.Parameter(
            displayName="Project name",
            name="in_ppp",
            datatype="GPstring",
            parameterType="Required",
            direction="Input")
      
        
        
        parameters=[dirr,in_mtx,in_ppp]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):


        wspace=parameters[0].valueAsText
        ssTXT=parameters[1].valueAsText
        projekt=parameters[2].valueAsText
    
        env.workspace=wspace


        # okreslenie zmiennych z pliku TXT

        op=""
        L8date = ""   # data
        L8time = ""   # godzina
        L8sunelev=0.0 # wys. slonca w stopniach
        L8sundistance=0.0  # zmiana odleglosci do slonca 1.0 srednia
        L8name=["L8"]*12   # 12 kanalow - 1,2,3,4,5,6,7,8,9,10,11,maska
        L8rad=zeros((11,2),float) # 11 kanalow do obliczenia radiance  ML, AL
        L8ref=zeros((9,2),float)  #  9 kanslow do obliczenia reflectance MR, AR
        L8K=zeros((2,2),float)    # kanaly 10 i 11 do obliczenia brightnes temp K1,K2
        minrad=zeros((9,2),float)    # minimalna radiancja, korekta



        in_plik=open(ssTXT,"r")
        nr=0
        for line in in_plik.readlines():
            lista=line.split(',')
            if nr==0:
                op=lista[0]
            if nr>1 and nr<=8:  #L8name
                #print nr, lista[0]
                L8name[nr-2]=lista[0]
            if nr>9 and nr<=20:#L8 rad
                #print lista[0],nr
                L8rad[nr-10,0]=float(lista[0])
                L8rad[nr-10,1]=float(lista[1])
            if nr>21 and nr<=30:#L8 rad
                #print lista[0],nr
                L8ref[nr-22,0]=float(lista[0])
                L8ref[nr-22,1]=float(lista[1])
            if nr==32: #L8date    
                L8date=lista[0]
                L8date=L8date[0:10]  
            if nr==33: #L8time
                L8time=lista[0]
                L8time=L8time[0:5]+" GMT"
            if nr==34: #L8sunelev
                L8sunelev=float(lista[0])
            if nr==35: #L8sundistance
                L8sundistance=float(lista[0])
            if nr>36: #minrad
                minrad[nr-37,0]=float(lista[0])
                minrad[nr-37,1]=float(lista[1])
            
            nr+=1
            L8K[0,0]=L8rad[9,0];L8K[0,1]=L8rad[9,1]
            L8K[1,0]=L8rad[10,0];L8K[1,1]=L8rad[10,1]
        in_plik.close()

        #arcpy.CheckOutExtension("Spatial")
        env.overwriteOutput=True



        c1=Raster(L8name[0])
        c2=Raster(L8name[1])
        c3=Raster(L8name[2])
        c4=Raster(L8name[3])
        c5=Raster(L8name[4])
        c6=Raster(L8name[5])
        c7=Raster(L8name[6])

        # Krok1

        c1r=c1*L8rad[0,0]+L8rad[0,1]
        c2r=c2*L8rad[1,0]+L8rad[1,1]
        c3r=c3*L8rad[2,0]+L8rad[2,1]
        c4r=c4*L8rad[3,0]+L8rad[3,1]
        c5r=c5*L8rad[4,0]+L8rad[4,1]
        c6r=c6*L8rad[5,0]+L8rad[5,1]
        c7r=c7*L8rad[6,0]+L8rad[6,1]
        del c1;del c2;del c3;del c4;del c5;del c6;del c7

        # Krok2

    
        #ESUN=[2027.0,2067.0,1893.0,1603.0,972.6,245.0,79.72,399.7]  #Gisagmaps
        ESUN=[1719.0,1787.0,1746.0,1536.0,997.0,811.0,75.0,399.7]   # Univ Yale (bez 9 ch)

        d2=L8sundistance
        d2=d2*d2
        dz=90-L8sunelev
        raddz=radians(dz)
        COS=cos(raddz)

        atm1=(minrad[0,0]-0.01*ESUN[0]*COS/(pi*d2))
        atm2=(minrad[1,0]-0.01*ESUN[1]*COS/(pi*d2))
        atm3=(minrad[2,0]-0.01*ESUN[2]*COS/(pi*d2))
        atm4=(minrad[3,0]-0.01*ESUN[3]*COS/(pi*d2))
        atm5=(minrad[4,0]-0.01*ESUN[4]*COS/(pi*d2))
        atm6=(minrad[5,0]-0.01*ESUN[5]*COS/(pi*d2))
        atm7=(minrad[6,0]-0.01*ESUN[6]*COS/(pi*d2))


        if atm1<0: atm1=0
        if atm2<0: atm2=0
        if atm3<0: atm3=0
        if atm4<0: atm4=0
        if atm5<0: atm5=0
        if atm6<0: atm6=0
        if atm7<0: atm7=0


        f1=c1r-atm1
        f2=c2r-atm2
        f3=c3r-atm3
        f4=c4r-atm4
        f5=c5r-atm5
        f6=c6r-atm6
        f7=c7r-atm7

            
        del c1r;del c2r;del c3r;del c4r;del c5r;del c6r;del c7r

        # zapisywanie
        csss="L8ch1rad_"+L8date+"_"+projekt+".tif"
        f1.save(csss)
        csss="L8ch2rad_"+L8date+"_"+projekt+".tif"
        f2.save(csss)
        csss="L8ch3rad_"+L8date+"_"+projekt+".tif"
        f3.save(csss)
        csss="L8ch4rad_"+L8date+"_"+projekt+".tif"
        f4.save(csss)
        csss="L8ch5rad_"+L8date+"_"+projekt+".tif"
        f5.save(csss)
        csss="L8ch6rad_"+L8date+"_"+projekt+".tif"
        f6.save(csss)
        csss="L8ch7rad_"+L8date+"_"+projekt+".tif"
        f7.save(csss)

        del f1;del f2;del f3;del f4;del f5;del f6;del f7
        #arcpy.CheckInExtension("Spatial")
