import arcpy
import math
from arcpy import env
from arcpy.sa import*
from numpy import*

class ReflectanceAtmcor(object):
    """Oblicza radiancje z korekcja atmosferyczna."""
    def __init__(self):
        self.label = "Reflectance with atmospheric correction"
        self.description =  ("Calculate reflectance with DOS 1% atmospheric correction")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Define the tool parameters."""
        # Input parameter
        dirr = arcpy.Parameter(
            displayName="Workspace",
            name="dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_mtx = arcpy.Parameter(
            displayName="Meta data text file (from Pan-sharpened composit tool)",
            name="in_mtx",
            datatype="DETextfile",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_ppp = arcpy.Parameter(
            displayName="Project name",
            name="in_ppp",
            datatype="GPstring",
            parameterType="Required",
            direction="Input")
      
        
        
        parameters=[dirr,in_mtx,in_ppp]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):


        wspace=parameters[0].valueAsText
        ssTXT=parameters[1].valueAsText
        projekt=parameters[2].valueAsText
        # zmienne

        env.workspace=wspace


        # okreslenie zmiennych z pliku TXT

        op=""
        L8date = ""   # data
        L8time = ""   # godzina
        L8sunelev=0.0 # wys. slonca w stopniach
        L8sundistance=0.0  # zmiana odleglosci do slonca 1.0 srednia
        L8name=["L8"]*12   # 12 kanalow - 1,2,3,4,5,6,7,8,9,10,11,maska
        L8rad=zeros((11,2),float) # 11 kanalow do obliczenia radiance  ML, AL
        L8ref=zeros((9,2),float)  #  9 kanslow do obliczenia reflectance MR, AR
        L8K=zeros((2,2),float)    # kanaly 10 i 11 do obliczenia brightnes temp K1,K2
        minrad=zeros((9,2),float)    # minimalna radiancja


        # czytanie pliku TXT

        in_plik=open(ssTXT,"r")
        nr=0
        for line in in_plik.readlines():
            lista=line.split(',')
            if nr==0:
                op=lista[0]
            if nr>1 and nr<=8:  #L8name
                #print nr, lista[0]
                L8name[nr-2]=lista[0]
            if nr>9 and nr<=20:#L8 rad
                #print lista[0],nr
                L8rad[nr-10,0]=float(lista[0])
                L8rad[nr-10,1]=float(lista[1])
            if nr>21 and nr<=30:#L8 rad
                #print lista[0],nr
                L8ref[nr-22,0]=float(lista[0])
                L8ref[nr-22,1]=float(lista[1])
            if nr==32: #L8date    
                L8date=lista[0]
                L8date=L8date[0:10]  
            if nr==33: #L8time
                L8time=lista[0]
                L8time=L8time[0:5]+" GMT"
            if nr==34: #L8sunelev
                L8sunelev=float(lista[0])
            if nr==35: #L8sundistance
                L8sundistance=float(lista[0])
            if nr>36: #minrad
                minrad[nr-37,0]=float(lista[0])
                minrad[nr-37,1]=float(lista[1])
            
            nr+=1
            L8K[0,0]=L8rad[9,0];L8K[0,1]=L8rad[9,1]
            L8K[1,0]=L8rad[10,0];L8K[1,1]=L8rad[10,1]
        in_plik.close()

        #arcpy.CheckOutExtension("Spatial")
        env.overwriteOutput=True



        c1=Raster(L8name[0])
        c2=Raster(L8name[1])
        c3=Raster(L8name[2])
        c4=Raster(L8name[3])
        c5=Raster(L8name[4])
        c6=Raster(L8name[5])
        c7=Raster(L8name[6])


        d2=L8sundistance
        d2=d2*d2
        dz=90-L8sunelev
        raddz=radians(dz)
        COS=cos(raddz)        

        # Krok1

        c1r=((c1*0.00002)-0.1)/COS
        c2r=((c2*0.00002)-0.1)/COS
        c3r=((c3*0.00002)-0.1)/COS
        c4r=((c4*0.00002)-0.1)/COS
        c5r=((c5*0.00002)-0.1)/COS
        c6r=((c6*0.00002)-0.1)/COS
        c7r=((c7*0.00002)-0.1)/COS
        del c1;del c2;del c3;del c4;del c5;del c6;del c7
        # Krok2

        sdn1=((minrad[0,1]*0.00002)-0.1)/COS
        sdn2=((minrad[1,1]*0.00002)-0.1)/COS
        sdn3=((minrad[2,1]*0.00002)-0.1)/COS
        sdn4=((minrad[3,1]*0.00002)-0.1)/COS
        sdn5=((minrad[4,1]*0.00002)-0.1)/COS
        sdn6=((minrad[5,1]*0.00002)-0.1)/COS
        sdn7=((minrad[6,1]*0.00002)-0.1)/COS
    
        # Krok3
        if sdn1<0.01: sdn1=0.01
        if sdn2<0.01: sdn2=0.01
        if sdn3<0.01: sdn3=0.01
        if sdn4<0.01: sdn4=0.01
        if sdn5<0.01: sdn5=0.01
        if sdn6<0.01: sdn6=0.01
        if sdn7<0.01: sdn7=0.01
        
        if sdn1>0.01: sdn1=sdn1-0.01
        if sdn2>0.01: sdn2=sdn2-0.01
        if sdn3>0.01: sdn3=sdn3-0.01
        if sdn4>0.01: sdn4=sdn4-0.01
        if sdn5>0.01: sdn5=sdn5-0.01
        if sdn6>0.01: sdn6=sdn6-0.01
        if sdn7>0.01: sdn7=sdn7-0.01

        f1=c1r-sdn1
        f2=c2r-sdn2
        f3=c3r-sdn3
        f4=c4r-sdn4
        f5=c5r-sdn5
        f6=c6r-sdn6
        f7=c7r-sdn7
        del c1r;del c2r;del c3r;del c4r;del c5r;del c6r;del c7r
        # zapisywanie
        csss="L8ch1ref_"+L8date+"_"+projekt+".tif"
        f1.save(csss)
        csss="L8ch2ref_"+L8date+"_"+projekt+".tif"
        f2.save(csss)
        csss="L8ch3ref_"+L8date+"_"+projekt+".tif"
        f3.save(csss)
        csss="L8ch4ref_"+L8date+"_"+projekt+".tif"
        f4.save(csss)
        csss="L8ch5ref_"+L8date+"_"+projekt+".tif"
        f5.save(csss)
        csss="L8ch6ref_"+L8date+"_"+projekt+".tif"
        f6.save(csss)
        csss="L8ch7ref_"+L8date+"_"+projekt+".tif"
        f7.save(csss)
        del f1;del f2;del f3;del f4;del f5;del f6;del f7

        #arcpy.CheckInExtension("Spatial")
