import arcpy
import math
from arcpy import env
from arcpy.sa import*
from numpy import*
import os

class SurfaceTemperature(object):
    """Oblicza temperature z zstoswaniem korekcji emisji."""
    def __init__(self):
        self.label = "Surface Temperature with emissivity correction"
        self.description =  ("Calculate surface temperature from channel 10 using emisivity correction.")
        self.canRunInBackground = False
    def getParameterInfo(self):
        """Defdef __init__(self):
        self.label = "Surface temperature emisivity corected"
        self.description =  ("Calculate surface temperature from channel 10 using emisivity correction.")
        self.canRunInBackground = Falseine the tool parameters."""
        # Input parameter
        dirr = arcpy.Parameter(
            displayName="Workspace",
            name="dirr",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")
        # Input parameter
        in_mtl = arcpy.Parameter(
            displayName="Meta data Landsat 8 MTL text file",
            name="in_mtl",
            datatype="DETextfile",
            parameterType="Required",
            direction="Input")
# Input parameter
        rff = arcpy.Parameter(
            displayName="BQA masking off",
            name="rff",
            datatype="GPBoolean",
            parameterType="Optional",
            direction="Input")
        rff.value=False        
        parameters=[dirr,in_mtl,rff]
        return parameters
    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return
    def execute(self, parameters, messages):

        wspace=parameters[0].valueAsText
        ssMTL=parameters[1].valueAsText
        bkodff=parameters[2].value



        env.workspace=wspace

        # okreslenie zmiennych z pliku MTL

        L8date = ""   # data
        L8time = ""   # godzina
        L8sunelev=0.0 # wys. slonca w stopniach
        L8sundistance=0.0  # zmiana odleglosci do slonca 1.0 srednia
        L8name=["L8"]*12   # 12 kanalow - 1,2,3,4,5,6,7,8,9,10,11,maska
        L8rad=zeros((11,2),float) # 11 kanalow do obliczenia radiance  ML, AL
        L8ref=zeros((9,2),float)  #  9 kanslow do obliczenia reflectance MR, AR
        L8K=zeros((2,2),float)    # kanaly 10 i 11 do obliczenia brightnes temp K1,K2

        # czytanie pliku MTL
        nne=0
        irad=0
        irad2=0
        nnra1=0
        nnra2=0
        nnrb1=0
        nnrb2=0
        nnrc1=0
        nnrc2=0
       
        in_plik=open(ssMTL,"r")

        for line in in_plik.readlines():
            lista=line.split(' ')
            #print "X"
            if len(lista) >4 and lista[4]=='DATE_ACQUIRED': #L8date
                L8date=lista[6]
                L8date=L8date[0:10]
            if len(lista) >4 and lista[4]=='SCENE_CENTER_TIME':  #L8time      
                L8time=lista[6]
                L8time=L8time[0:5]+" GMT"
            if len(lista) >4 and lista[4]=='SUN_ELEVATION':   #L8sunelev      
                L8sunelev=float(lista[6])
            if len(lista) >4 and lista[4]=='EARTH_SUN_DISTANCE': #L8sundistance       
                L8sundistance=float(lista[6])
            if len(lista) >4 and lista[4]=='FILE_NAME_BAND_1':  #L8name 1     
                L8=lista[6]
                nn=len(L8)
                L8name[nne]=L8[1:nn-2]
                nne+=1
            if nne>0 and nne< 12 and lista[4]<>'FILE_NAME_BAND_1': #L8name 2-11
                L8=lista[6]
                nn=len(L8)
                L8name[nne]=L8[1:nn-2]
                nne+=1
            if irad>0 and irad< 41: # read rad and ref rescaling coeff
                if irad<12:
                    L8rad[nnra1,0]=float(lista[6])
                    nnra1+=1
                if irad>=12 and irad<23:
                    L8rad[nnra2,1]=float(lista[6])
                    nnra2+=1
                if irad>=23 and irad<32:
                    L8ref[nnrb1,0]=float(lista[6])
                    nnrb1+=1
                if irad>=32 and irad<41:
                    L8ref[nnrb2,1]=float(lista[6])
                    nnrb2+=1
                irad+=1    
            if irad2>0 and irad2<5: # read thermal constants
                if irad2<3:
                    L8K[nnrc1,0]=float(lista[6])
                    nnrc1+=1
                if irad2>=3:
                    L8K[nnrc2,1]=float(lista[6])
                    nnrc2+=1
                irad2+=1    
            if len(lista) >3 and lista[4]=='RADIOMETRIC_RESCALING\n' and irad==0:  # start reading radiometric
                irad=1;
            if len(lista) >3 and lista[4]=='TIRS_THERMAL_CONSTANTS\n' and irad2==0:  # start reading termal co.
                irad2=1;


        in_plik.close()
        #arcpy.CheckOutExtension("Spatial")
        env.overwriteOutput=True



        c10=Raster(L8name[9])
        c11=Raster(L8name[10])




        # Obliczenie radiancji


        c10r=c10*L8rad[9,0]+L8rad[9,1]
        c11r=c11*L8rad[10,0]+L8rad[10,1]

        # Obliczenie emisyjnosci z NDVI

        c4=Raster(L8name[3])
        c5=Raster(L8name[4])
        c4r=c4*L8rad[3,0]+L8rad[3,1]
        c5r=c5*L8rad[4,0]+L8rad[4,1]
        ndvi=(c5r-c4r)/(c5r+c4r)
        ndvi2=Con((ndvi>=0.145) & (ndvi<0.728),ndvi,0.5)
        ndvitocal=1.0094+0.047*(Ln(ndvi2)) 
        remap=RemapRange([[-1.0,-0.18,1],[-0.18,0.145,2],[0.145,0.728,3],[0.728,1.0,4]])
        ndvireclas2=Reclassify(ndvi,"VALUE",remap,"NODATA")
        ndvireclas=Expand(ndvireclas2,1,[1])
        emisa=Con(ndvireclas==3,ndvitocal,Con(ndvireclas==1,0.985,Con(ndvireclas==2,0.919,Con(ndvireclas==4,0.994))))
        emis=Con(IsNull(emisa),0.995,emisa)
        # Obliczenie maski chmur z BQA
        cbqa=Raster(L8name[11])
        if bkodff==0:
            maska=Con((cbqa!=53248)&(cbqa!=56320) & (cbqa!=57344)& (cbqa!=59424) & (cbqa!=61440),1)
        
        # Obliczenie brightnes temperature


        #c10t=(L8K[0,1]/(Ln((L8K[0,0]/(c10r/emis))+1.0)))
        
        c10t=(L8K[0,1]/(Ln((L8K[0,0]/(c10r))+1.0)))
        c10ts=c10t/(1+(10.8*c10t/14380*Ln(emis)))
        if bkodff==1: maska=1.0
        c10tt=c10ts*maska-273.16
        c10ttt=Con(c10tt>-30,c10tt)
        

        # zapisywanie

        csss="L8ST_"+L8date+".tif"
        c10ttt.save(csss)

        del c4;del c5;del c10;del c11;del cbqa
        #arcpy.CheckInExtension("Spatial")
